from threading import Event, Thread
from Tkinter import Tk
from vtk import vtkRenderer
from vtk.tk.vtkTkRenderWidget import vtkTkRenderWidget

class Visualiser(Thread):
    """Superclass of both the realtime and offline VTK visualisers
    """
    def __init__(self, domain):
        Thread.__init__(self)

        self.domain = domain
        self.running = True

        # Synchronisation Constructs
        self.sync_idle = Event()
        self.sync_idle.clear()
        self.sync_unpaused = Event()
        self.sync_unpaused.set()
        self.sync_redrawReady = Event()
        self.sync_redrawReady.clear()

        # Structures for Height Based quantities
        self.height_quantities = []
        self.height_zScales = {}
        self.height_dynamic = {}

    def run(self):
        self.setup_gui()
        self.tk_root.after(100, self.sync_idle.set)
        self.tk_root.mainloop()

    # --- Height Based Rendering --- #

    def render_quantity_height(self, quantityName, zScale=1.0, dynamic=True):
        """Instruct the visualiser to render a quantity
        using the value at a point as its height.
        The value at each point is multiplied by z_scale,
        and if dynamic=False, the quantity is not recalculated
        on each update.
        """
        if quantityName in self.domain.quantities:
            self.height_quantities.append(quantityName)
            self.height_zScales[quantityName] = zScale
            self.height_dynamic[quantityName] = dynamic

    # --- Colour Coding --- #

    # --- Vector Fields --- #

    # --- GUI Setup --- #

    def setup_gui(self):
        self.tk_root = Tk()
        self.tk_root.title("Visualisation")
        self.tk_root.after(100, self.redraw)
        self.tk_root.bind("<Destroy>", self.destroyed)

        self.tk_renderWidget = vtkTkRenderWidget(self.tk_root, width=400, height=400)
        self.tk_renderWidget.grid(row=0, column=0)
        self.vtk_renderer = vtkRenderer()
        self.tk_renderWidget.GetRenderWindow().AddRenderer(self.vtk_renderer)

    # --- GUI Events --- #

    def destroyed(self, event):
        if event.widget == self.tk_root:
            self.shutdown()

    def redraw(self):
        pass

    def shutdown(self):
        self.running = False
        self.sync_idle.set()
        self.sync_unpaused.set()
        self.tk_root.withdraw()
        self.tk_root.quit()
